﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

namespace STMBench7.Compatability
{
    /**
     * The Java version used TreeSets, which aren't here
     * C# has JustASets, which Bartok doesn't provide 
     * so we'll try this.
     */
    public class JustASet<T> : IEnumerable<T>
    {
        //private LinkedList<T> impl = new LinkedList<T>();
        private Hashtable impl = new Hashtable();

        public JustASet() { }
        public JustASet(IEnumerable<T> c)
        {
            foreach (T e in c) Add(e);
        }

        public bool Add(T e)
        {
            if (impl.ContainsKey(e)) return false;
            impl.Add(e, 7777);
            return true;
        }
        public bool Remove(T e)
        {
            if (!impl.ContainsKey(e)) return false;
            impl.Remove(e);
            return true;
        }
        public bool Contains(T e)
        {
            return impl.ContainsKey(e);
        }

   
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return impl.GetEnumerator();
        }

        public int Count
        {
            get
            {
                return impl.Count;
            }
        }

        IEnumerator<T> IEnumerable<T>.GetEnumerator()
        {
            return new CastingEnumerator<T>(impl.GetEnumerator());
        }
    }

    class CastingEnumerator<T> : IEnumerator<T>
    {
        IEnumerator e;
        public CastingEnumerator(IEnumerator e){
            this.e=e;
        }


        #region IEnumerator<T> Members

        public T Current
        {
            get { return (T)((DictionaryEntry)e.Current).Key; }
        }

        #endregion

        #region IDisposable Members

        public void Dispose()
        {
            //do nothing
        }

        #endregion

        #region IEnumerator Members

        object IEnumerator.Current
        {
            get { return ((DictionaryEntry)e.Current).Key; }
        }

        public bool MoveNext()
        {
            return e.MoveNext();
        }

        public void Reset()
        {
            e.Reset();
        }

        #endregion
    }
}
