﻿using System;
 
 
using System.Text;

using STMBench7.Annotations;
using STMBench7.Backend;
using STMBench7.Operations;

namespace STMBench7.Core
{
	/**
	 * Used to create and destroy atomic parts while
	 * maintaining the consistency of the data structure
	 * and the indexes.
	 */
	[Immutable]
	public class AtomicPartBuilder : DesignObjBuilder {

		private readonly IdPool idPool;

		private readonly Index/*<IntIndexKey,AtomicPart>*/ partIdIndex;
		private readonly Index/*<IntIndexKey,LargeSet<AtomicPart>>*/ partBuildDateIndex;

		public AtomicPartBuilder(Index/*<IntIndexKey,AtomicPart>*/ partIdIndex,
				Index/*<IntIndexKey,LargeSet<AtomicPart>>*/ partBuildDateIndex) {
			
			this.partIdIndex = partIdIndex;
			this.partBuildDateIndex = partBuildDateIndex;
			idPool = BackendFactory.instance.CreateIdPool(Parameters.MaxAtomicParts);
		}

        public virtual AtomicPart CreateAndRegisterAtomicPart()
        {
			int id = idPool.GetId();
			String type = CreateType();
			int buildDate = CreateBuildDate(Parameters.MinAtomicDate, Parameters.MaxAtomicDate);
			int x = ThreadRandom.NextInt(Parameters.XYRange), 
				y = x + 1; // convenient for invariant tests
			
			AtomicPart part = designObjFactory.CreateAtomicPart(id, type, buildDate, x, y);

			partIdIndex.Put(new IntIndexKey(id), part);
			BaseOperation.AddAtomicPartToBuildDateIndex(partBuildDateIndex, part);
		
			return part;
		}

        public virtual void UnregisterAndRecycleAtomicPart(AtomicPart part)
        {
			int partId = part.GetId();

			BaseOperation.RemoveAtomicPartFromBuildDateIndex(partBuildDateIndex, part);
			partIdIndex.Remove(new IntIndexKey(partId));
			part.ClearPointers();
			
			idPool.PutUnusedId(partId);
		}
	}

}
