﻿using System;
 
 
using System.Text;

using STMBench7.Annotations;

namespace STMBench7.Core
{
	/**
	 * A factory for creating all the objects of the main benchmark
	 * data structure. A default implementation is provided by the
	 * stmbench7.impl.core.DesignObjFactoryImpl class, but most STM
	 * implementations will provide their own implementation, which
	 * will create transaction-safe versions of the data structure
	 * elements.
	 */
	[Immutable]
	public abstract class DesignObjFactory {

		public static DesignObjFactory instance = null;
		
		public static void SetInstance(DesignObjFactory newInstance) {
			if(instance != null)
				throw new RuntimeError("DesignObjFactory already instantiated");
			instance = newInstance;
		}
		
		public abstract AtomicPart CreateAtomicPart(int id, String type, int buildDate, int x, int y);
		public abstract Connection CreateConnection(AtomicPart from, AtomicPart to, String type, int length);
		public abstract BaseAssembly CreateBaseAssembly(int id, String type, int buildDate, 
				Module module, ComplexAssembly superAssembly);
		public abstract ComplexAssembly CreateComplexAssembly(int id, String type, int buildDate, 
				Module module, ComplexAssembly superAssembly);
		public abstract CompositePart CreateCompositePart(int id, String type, int buildDate, 
				Document documentation);
		public abstract Document CreateDocument(int id, String title, String text);
		public abstract Manual CreateManual(int id, String title, String text);
		public abstract Module CreateModule(int id, String type, int buildDate, Manual man);	
	}

}
