﻿using System;
 
 
using System.Text;

using STMBench7.Annotations;
using STMBench7.Backend;

namespace STMBench7.Core
{

	/**
	 * Used to create and destroy document elements while
	 * maintaining the consistency of the data structure
	 * and the indexes.
	 */
	[Immutable]
	public class DocumentBuilder : DesignObjBuilder {

		private readonly IdPool idPool;
		private readonly Index/*<StringIndexKey,Document>*/ documentTitleIndex;
		
		public DocumentBuilder(Index/*<StringIndexKey,Document>*/ documentTitleIndex) {
			this.documentTitleIndex = documentTitleIndex;
			idPool = BackendFactory.instance.CreateIdPool(Parameters.MaxCompParts);
		}

        public virtual Document CreateAndRegisterDocument(int compositePartId)
        {
			int docId = idPool.GetId();
			String docTitle = "Composite Part #" + compositePartId;
			String docText = CreateText(Parameters.DocumentSize, 
						      "I am the documentation for composite part #" + compositePartId + "\n");

			Document documentation = designObjFactory.CreateDocument(docId, docTitle, docText);
			documentTitleIndex.Put(new StringIndexKey(docTitle), documentation);
			
			return documentation;
		}

        public virtual void UnregisterAndRecycleDocument(Document document)
        {
			document.SetPart(null);
			documentTitleIndex.Remove(new StringIndexKey(document.GetTitle()));
			idPool.PutUnusedId(document.GetDocumentId());
		}
	}

}
