﻿using System;
 
 
using System.Text;

using STMBench7.Annotations;
using STMBench7.Backend;

namespace STMBench7.Core
{
	/**
	 * Used to create a module element with the default initial 
	 * data structure underneath. Calls other builder classes
	 * to construct respective elements of the structure.
	 */
	[Immutable]
	public class ModuleBuilder : DesignObjBuilder {

		private readonly IdPool idPool;
		private readonly ManualBuilder manualFactory;
		private readonly AssemblyBuilder assemblyBuilder;
		
		public ModuleBuilder(Index/*<IntIndexKey,BaseAssembly>*/ baseAssemblyIdIndex,
				Index/*<IntIndexKey,ComplexAssembly>*/ complexAssemblyIdIndex) {
			manualFactory = new ManualBuilder();
			assemblyBuilder = new AssemblyBuilder(baseAssemblyIdIndex,
					complexAssemblyIdIndex);
			idPool = BackendFactory.instance.CreateIdPool(Parameters.NumModules);
		}

        public virtual Module CreateRegisterModule()
        {
			int moduleId = idPool.GetId();
			Manual manual = manualFactory.CreateManual(moduleId);
			String type = CreateType();
			int buildDate = CreateBuildDate(Parameters.MinModuleDate, Parameters.MaxModuleDate);
			
			Module module = designObjFactory.CreateModule(moduleId, type, buildDate, manual);
			
			ComplexAssembly designRoot = (ComplexAssembly) assemblyBuilder.CreateAndRegisterAssembly(module, null);
			module.SetDesignRoot(designRoot);
			
			return module;
		}

        public virtual AssemblyBuilder GetAssemblyBuilder()
        {
			return assemblyBuilder;
		}
	}

}
