﻿using System;
 
 
using System.Text;

using STMBench7.Core;
using STMBench7.Annotations;

namespace STMBench7.Correctness.Invariants
{
	/**
	 * Performs the check of data structure invariants. The operation is always
	 * performed in a single thread, without any concurrency, and so it does not
	 * have to be thread-safe. However, it is executed by an OperationExecutor, as
	 * any other operation.
	 */
	[Immutable]
	[ThreadLocal]
	public class CheckInvariantsOperation : Operation {

		private readonly Setup setup;
		private readonly bool initial;

		public CheckInvariantsOperation(Setup setup, bool initial) {
			this.setup = setup;
			this.initial = initial;
		}

		[ReadOnly]
		public int PerformOperation() {
            
			TraversedObjects traversedObjects = new TraversedObjects();
			ModuleTest.CheckInvariants(setup.GetModule(), initial, traversedObjects);
			Console.Error.WriteLine("\nChecking indexes...");
			IndexTest.CheckInvariants(setup, initial, traversedObjects);
			Console.Error.WriteLine("\nInvariants OK.");
			return 0;
		}

		public OperationId GetOperationId() {
			return null;
		}
	}

}
