﻿using System;
 
 
using System.Text;

using STMBench7.Annotations;
using STMBench7.Core;

namespace STMBench7.Correctness.Invariants
{
	/**
	 * Test of invariants of a complex assembly.
	 */
	[Immutable]
	[ThreadLocal]
	public class ComplexAssemblyTest : InvariantTest {

		public static void CheckInvariants(ComplexAssembly assembly, bool initial, ComplexAssembly parentAssembly,
				Module module, TraversedObjects traversedObjects) {
			
			traversedObjects.complexAssemblies.Add(assembly);
			
			AssemblyTest.CheckInvariants(assembly, initial, Parameters.MaxComplexAssemblies, parentAssembly, module);
			
			int id = assembly.GetId(), level = assembly.GetLevel();
			if(level <= 1 || level > Parameters.NumAssmLevels)
				ReportError(assembly, id, "level", 1, Parameters.NumAssmLevels, level);
			
			foreach(Assembly subAssembly in assembly.GetSubAssemblies()) {
				if(level > 2) {
					if(! (subAssembly is ComplexAssembly))
						ReportError(assembly, id, "subAssembly not of type ComplexAssembly at level = " + level);
					ComplexAssemblyTest.CheckInvariants((ComplexAssembly)subAssembly, initial, assembly, module, traversedObjects);
				}
				else {
					if(! (subAssembly is BaseAssembly))
						ReportError(assembly, id, "subAssembly not of type BaseAssembly at level = 2");
					BaseAssemblyTest.CheckInvariants((BaseAssembly)subAssembly, initial, assembly, module, traversedObjects);
				}
			}
				
		}

	}

}
