﻿using System;
 
 
using System.Text;

using STMBench7.Annotations;
using STMBench7.Core;

namespace STMBench7.Correctness.Invariants
{
	/**
	 * Helper methods for all invariant tests.
	 */
	[Immutable]
	[ThreadLocal]
	public class InvariantTest {

		protected static void ReportError(Object obj, int id, String message) {
			String withId = "";
			if(id > 0) withId = " with id = " + id;
			throw new RuntimeError("Invariant violated! Object " + obj.GetType().Name + 
					withId + ": " + message);
		}
		
		protected static void ReportError(Object obj, int id, String attribute, int expected, int found) {
			ReportError(obj, id, "attribute " + attribute + 
					" --> expected value = " + expected + ", found = " + found);
		}
		
		protected static void ReportError(Object obj, int id, String attribute, int expectedMin, int expectedMax, int found) {
			ReportError(obj, id, "attribute " + attribute + 
					" --> expected value in [" + expectedMin + "," + expectedMax + "], found = " + found);
		}
		
		protected static void ReportError(Object obj, int id, String attribute, String expected, String found) {
			ReportError(obj, id, "attribute " + attribute + 
					" --> expected value = " + expected + ", found = " + found);
		}
	}

}
