﻿using System;
 
 
using System.Text;
using STMBench7.Annotations;
using STMBench7.Backend;
using STMBench7.Core;
using STMBench7.Compatability;

namespace STMBench7.Impl.Backend
{
	/**
	 * Used to generate ids of various elements.
	 * This default implementation is NOT thread-safe.
	 */
	public class IdPoolImpl : IdPool {

	    protected LinkedList2 idPool;

        [BartokSTMExtensions.NoLogTrans] //allocation that does not escape "this"
	    public IdPoolImpl(int maxNumberOfIds) {
	    	idPool = new LinkedList2();
	    	for(int id = 1; id <= maxNumberOfIds; id++) {
	    		//idPool.Offer(id);
                idPool.AddLast(id);
	    	}
	    }

	    [Update]
	    public int GetId() {
	    	//int id = idPool.Poll();
            //if(id == null) throw new OperationFailedException();

            if (idPool.First == null) throw new OperationFailedException();

            int id = (int)idPool.First.Value;
            idPool.RemoveFirst();	
	    	return id;
	    }
	    
	    [Update]
	    public void PutUnusedId(int id) {
	    	//idPool.Offer<int>(id);
            idPool.AddLast(id);
	    }
	    
	    public override String ToString() {
	    	String txt = "IdPool:";
	    	foreach(int id in idPool) txt += " " + id;
	    	return txt;
	    }
	}

}
