﻿using System;
 
using System.Collections;
 
using System.Text;
using STMBench7.Backend;
using STMBench7.Core;
using STMBench7.Compatability;

namespace STMBench7.Impl.Backend
{
	/**
	 * A simple implementation of an index
	 * (NOT thread-safe).
	 */
	public class TreeMapIndex/*<K,V>*/ : Index/*<K,V> where K: IndexKey*/{

        //protected TreeMap<K,V> index;
        protected SortedList index;


        [BartokSTMExtensions.NoLogTrans] //allocation that does not escape "this"
		public TreeMapIndex() {
            //index = new TreeMap<K, V>();
            index = new SortedList();
        }

        public void Put(Object key, Object value)
        {
			if(value == null) throw new RuntimeError("TreeMapIndex does not support null values!");
			//index.put(key, value);
            if (index.ContainsKey(key))
            {
                index[key] = value;
                return;
            }
            index.Add(key, value);
		}

        public Object PutIfAbsent(Object key, Object value)
        {
			if(value == null) throw new RuntimeError("TreeMapIndex does not support null values!");
			
            if(index.ContainsKey(key)) return index[key];
            index.Add(key, value);
            return value;
		}

        public Object Get(Object key)
        {
            try
            {
                return index[key];
            }
            catch (Exception) //ie, not found
            {
                return null; //as per Java's TreeMap behavior
            }
		}

        public IEnumerable GetRange(IndexKey minKey, IndexKey maxKey)
        {
            /*SortedDictionary<K, V> sub = new SortedDictionary<K, V>();
            foreach (K key in index.Keys)
            {
                if (!key.GreaterThan(minKey) && key.LowerThan(maxKey)) sub.Add(key, index[key]);
            }
            return sub.Values;*/

            
            LinkedList2 sub = new LinkedList2();
            foreach (IndexKey key in index.Keys)
            {
                if (!key.GreaterThan(minKey) && key.LowerThan(maxKey)) sub.Add(index[key]);
            }
            return sub; //.Cast<V>();

            

			//return index.subMap(minKey, maxKey).values();
		}

        public bool Remove(Object key)
        {
			//V removedValue = index.Remove(key);
			//return (removedValue != null);

            
            //return index.Remove(key);

            if (index.ContainsKey(key))
            {
                index.Remove(key);
                return true;
            }
            return false;
		}

        public IEnumerator Iterator() {
            return index.Values.GetEnumerator();
		}

        IEnumerator IEnumerable.GetEnumerator()
        {
            return index.Values.GetEnumerator();
        }
    }

}
