﻿using System;
 
 
using System.Text;

using STMBench7.Core;
using STMBench7.Backend;
using STMBench7.Impl.Backend;

namespace STMBench7.Impl.Core
{

	/**
	 * STMBench7 benchmark Atomic Part (see the specification).
	 * Default implementation.
	 */
	public class AtomicPartImpl : DesignObjImpl , AtomicPart {

	    private int x, y;
	    private SmallSetImpl/*<Connection>*/ to, from;
	    private CompositePart partOf;

        public AtomicPartImpl(int id, String type, int buildDate, int x, int y)
            : base(id, type, buildDate)
        {
	    	this.x = x;
	    	this.y = y;
	    	to = new SmallSetImpl/*<Connection>*/();
	    	from = new SmallSetImpl/*<Connection>*/();
	    }

        public AtomicPartImpl(AtomicPartImpl source)
            : base(source)
        {
	    	this.x = source.x;
	    	this.y = source.y;
	    	this.to = new SmallSetImpl/*<Connection>*/(source.to);
	    	this.from = new SmallSetImpl/*<Connection>*/(source.from);
	    	this.partOf = source.partOf;
	    }

        public virtual void ConnectTo(AtomicPart destination, String type, int length)
        {
	    	Connection connection = new ConnectionImpl(this, destination, type, length);
	    	to.Add(connection);
	    	destination.AddConnectionFromOtherPart(connection.GetReversed());
	    }

        public virtual void AddConnectionFromOtherPart(Connection connection)
        {
	    	from.Add(connection);
	    }

        public virtual void SetCompositePart(CompositePart partOf)
        {
	    	this.partOf = partOf;
	    }

        public virtual int GetNumToConnections()
        {
	    	return to.Size();
	    }

        public virtual ImmutableCollection/*<Connection>*/ GetToConnections()
        {
	    	return new ImmutableCollectionImpl/*<Connection>*/(to);
	    }

        public virtual ImmutableCollection/*<Connection>*/ GetFromConnections()
        {
	    	return new ImmutableCollectionImpl/*<Connection>*/(from);
	    }

        public virtual CompositePart GetPartOf()
        {
	    	return partOf;
	    }

        public virtual void SwapXY()
        {
	    	int tmp = y;
	    	y = x;
	    	x = tmp;
	    }

        public virtual int GetX()
        {
	    	return x;
	    }

        public virtual int GetY()
        {
	    	return y;
	    }

        public virtual void ClearPointers()
        {
	    	x = y = 0;
	    	to = null;
	    	from = null;
	    	partOf = null;
	    }

        public virtual int CompareTo(Object/*AtomicPart*/ part)
        {
            AtomicPart a = (AtomicPart)part;
			return id - a.GetId();

            
		}

        public override int GetHashCode()
        {
            return id;
        }
        
	}

}
