﻿using System;
 
 
using System.Text;

using STMBench7.Core;
using STMBench7.Backend;
using STMBench7.Compatability;
using STMBench7.Impl.Backend;

namespace STMBench7.Impl.Core
{
	/**
	 * STMBench7 benchmark Complex Assembly (see the specification).
	 * Default implementation.
	 */
	public class ComplexAssemblyImpl : AssemblyImpl , ComplexAssembly {

	    private SmallSetImpl/*<Assembly>*/ subAssemblies;
	    private readonly short level;

        public ComplexAssemblyImpl(int id, String type, int buildDate, Module module, ComplexAssembly superAssembly)
            : base(id, type, buildDate, module, superAssembly)
        {
	    	subAssemblies = new SmallSetImpl/*<Assembly>*/();
	    	
	       	if(superAssembly == null) level = (short)Parameters.NumAssmLevels;
	    	else level = (short)(superAssembly.GetLevel() - 1);
	    }

        public ComplexAssemblyImpl(ComplexAssemblyImpl source)
            : base(source)
        {
	    	this.subAssemblies = new SmallSetImpl/*<Assembly>*/(subAssemblies);
	    	this.level = source.level;
	    }

        public virtual bool AddSubAssembly(Assembly assembly)
        {
	    	if(assembly is BaseAssembly && level != 2)
	    		throw new RuntimeError("ComplexAssembly.AddAssembly: BaseAssembly at wrong level!");
	    	
	    	bool notAddedBefore = subAssemblies.Add(assembly);
	    	return notAddedBefore;
	    }

        public virtual bool RemoveSubAssembly(Assembly assembly)
        {
	    	return subAssemblies.Remove(assembly);
	    }

        public virtual ImmutableCollection/*<Assembly>*/ GetSubAssemblies()
        {
	    	return new ImmutableCollectionImpl/*<Assembly>*/(subAssemblies);
	    }

        public virtual short GetLevel()
        {
	    	return level;
	    }
	    
        public override void ClearPointers()
        {
	    	base.ClearPointers();
	    	subAssemblies = null;
	    }
	}

}
