﻿using System;
 
 
using System.Text;

using STMBench7.Core;
using STMBench7.Backend;
using STMBench7.Impl.Backend;

namespace STMBench7.Impl.Core
{
	/**
	 * STMBench7 benchmark Composite Part (see the specification).
	 */
	public class CompositePartImpl : DesignObjImpl , CompositePart {

	    private Document documentation;
	    private BagImpl/*<BaseAssembly>*/ usedIn;
	    private LargeSet/*<AtomicPart>*/ parts;
	    private AtomicPart rootPart;

        public CompositePartImpl(int id, String type, int buildDate, Document documentation)
            : base(id, type, buildDate)
        {
	    	this.documentation = documentation;
	    	documentation.SetPart(this);
	    	usedIn = new BagImpl/*<BaseAssembly>*/();
	    	parts = BackendFactory.instance.CreateLargeSet/*<AtomicPart>*/();
	    }

	    public CompositePartImpl(CompositePartImpl source) :base(source) {
	    	this.documentation = source.documentation;
	    	this.usedIn = new BagImpl/*<BaseAssembly>*/(source.usedIn);
	    	this.parts = source.parts;
	    	this.rootPart = source.rootPart;
	    }

        public virtual void AddAssembly(BaseAssembly assembly)
        {
	    	usedIn.Add(assembly);
	    }

        public virtual bool AddPart(AtomicPart part)
        {
	    	bool notAddedBefore = parts.Add(part);
	    	if(! notAddedBefore) return false;

	    	part.SetCompositePart(this);
	    	if(rootPart == null) rootPart = part;

	    	return true;
	    }

        public virtual AtomicPart GetRootPart()
        {
	    	return rootPart;
	    }

        public virtual void SetRootPart(AtomicPart part)
        {
			rootPart = part;
		}

        public virtual Document GetDocumentation()
        {
	    	return documentation;
	    }

        public virtual LargeSet/*<AtomicPart>*/ GetParts()
        {
	    	return parts;
	    }

        public virtual void RemoveAssembly(BaseAssembly assembly)
        {
	    	usedIn.Remove(assembly);
	    }

        public virtual ImmutableCollection/*<BaseAssembly>*/ GetUsedIn()
        {
	    	return new ImmutableCollectionImpl/*<BaseAssembly>*/(usedIn);
	    }

        public virtual void ClearPointers()
        {
	    	documentation = null;
	    	parts = null;
	    	usedIn = null;
	    	rootPart = null;
	    }
	}

}
