﻿using System;
 
 
using System.Text;

using STMBench7.Annotations;
using STMBench7.Core;
using STMBench7.Compatability;

namespace STMBench7.Impl.Core
{

	/**
	 * Implements methods that create objects implementing
	 * interfaces defined in stmbench7.core. This default implementation 
	 * constructs objects that are NOT thread-safe. 
	 */
	public class DesignObjFactoryImpl : DesignObjFactory {

        public override AtomicPart CreateAtomicPart(int id, String type, int buildDate, int x, int y)
        {
			return new AtomicPartImpl(id, type, buildDate, x, y);
		}

        public override Connection CreateConnection(AtomicPart from, AtomicPart to,
				String type, int length) {
			return new ConnectionImpl(from, to, type, length);
		}

        public override BaseAssembly CreateBaseAssembly(int id, String type, int buildDate, 
				Module module, ComplexAssembly superAssembly) {
			return new BaseAssemblyImpl(id, type, buildDate, module, superAssembly);
		}

        public override ComplexAssembly CreateComplexAssembly(int id, String type, int buildDate, 
				Module module, ComplexAssembly superAssembly) {
			return new ComplexAssemblyImpl(id, type, buildDate, module, superAssembly);
		}
		
        public override CompositePart CreateCompositePart(int id, String type, int buildDate, 
				Document documentation) {
			return new CompositePartImpl(id, type, buildDate, documentation);
		}
		
        public override Document CreateDocument(int id, String title, String text)
        {
			return new DocumentImpl(id, title, text);
		}
		
        public override Manual CreateManual(int id, String title, String text)
        {
			return new ManualImpl(id, title, text);
		}
		
        public override Module CreateModule(int id, String type, int buildDate, Manual man)
        {
			return new ModuleImpl(id, type, buildDate, man);
		}
	}

}
