﻿using System;
 
 
using System.Text;

using STMBench7.Core;

namespace STMBench7.Impl.Core
{
	/**
	 * STMBench7 benchmark Document (see the specification).
	 * Default implementation.
	 */
	public class DocumentImpl : Document {

        private int hash = new Object().GetHashCode();

        public override int GetHashCode()
        {
            return hash;
        }
	    
	    private readonly int id;
	    private String title;
	    private String text;
	    private CompositePart part;

	    public DocumentImpl(int id, String title, String text) {
	    	this.id = id;
	    	this.title = title;
	    	this.text = text;
	    }

	    public DocumentImpl(DocumentImpl source) {
	    	this.title = source.title;
	    	this.id = source.id;
	    	this.text = source.text;
	    	this.part = source.part;
	    }

        public virtual void SetPart(CompositePart part)
        {
	    	this.part = part;
	    }

        public virtual CompositePart GetCompositePart()
        {
	    	return part;
	    }

        public virtual int GetDocumentId()
        {
	    	return id;
	    }

        public virtual String GetTitle()
        {
	    	return title;
	    }

        public virtual void NullOperation()
        {
	    }

        public virtual int SearchText(char symbol)
        {
	    	int occurences = 0;

           
	    	for(int i = 0; i < text.Length; i++)
	    		if(text[i] == symbol) occurences++;


	    	return occurences;
	    }

        public virtual int ReplaceText(String from, String to)
        {
            if (!text.StartsWith(from)) { return 0; }
            if (from.Equals(""))
            {
                text = to + text;
                return 1;
            }
            else
            {
                String tmp = text.Substring(from.Length);
                text = to + tmp;
                return 1;
            }
	    }

        public virtual bool TextBeginsWith(String prefix)
        {
			return text.StartsWith(prefix);
		}

        public virtual String GetText()
        {
			return text;
		}
	}

}
