﻿using System;
 
 
using System.Text;
using STMBench7.Annotations;
using STMBench7.Operations;

namespace STMBench7
{
	/**
	 * Lists all STMBench7 operations, together with the classes implementing them
	 * and their types.
	 */
	[NonAtomic]
	public /*enum*/ class OperationId {

        public static readonly OperationId
		T1 = new OperationId(typeof(Traversal1), OperationType.TRAVERSAL_RO, "T1"), 
		T2a = new OperationId(typeof(Traversal2a), OperationType.TRAVERSAL, "T2a"),
		T2b = new OperationId(typeof(Traversal2b), OperationType.TRAVERSAL, "T2b"),
		T2c = new OperationId(typeof(Traversal2c), OperationType.TRAVERSAL, "T2c"),
		T3a = new OperationId(typeof(Traversal3a), OperationType.TRAVERSAL, "T3a"),
		T3b = new OperationId(typeof(Traversal3b), OperationType.TRAVERSAL, "T3b"),
		T3c = new OperationId(typeof(Traversal3c), OperationType.TRAVERSAL, "T3c"),
		T4 = new OperationId(typeof(Traversal4), OperationType.TRAVERSAL_RO, "T4"),
		T5 = new OperationId(typeof(Traversal5), OperationType.TRAVERSAL, "T5"),
		T6 = new OperationId(typeof(Traversal6), OperationType.TRAVERSAL_RO, "T6"),
		Q6 = new OperationId(typeof(Query6), OperationType.TRAVERSAL_RO, "Q6"),
		Q7 = new OperationId(typeof(Query7), OperationType.TRAVERSAL_RO, "Q7"),
		
		ST1 = new OperationId(typeof(ShortTraversal1), OperationType.SHORT_TRAVERSAL_RO, "ST1"),
		ST2 = new OperationId(typeof(ShortTraversal2), OperationType.SHORT_TRAVERSAL_RO, "ST2"),
		ST3 = new OperationId(typeof(Traversal7), OperationType.SHORT_TRAVERSAL_RO, "ST3"),
		ST4 = new OperationId(typeof(Query4), OperationType.SHORT_TRAVERSAL_RO, "ST4"),
		ST5 = new OperationId(typeof(Query5), OperationType.SHORT_TRAVERSAL_RO, "ST5"),
		ST6 = new OperationId(typeof(ShortTraversal6), OperationType.SHORT_TRAVERSAL, "ST6"),
		ST7 = new OperationId(typeof(ShortTraversal7), OperationType.SHORT_TRAVERSAL, "ST7"),
		ST8 = new OperationId(typeof(ShortTraversal8), OperationType.SHORT_TRAVERSAL, "ST8"),
		ST9 = new OperationId(typeof(ShortTraversal9), OperationType.SHORT_TRAVERSAL_RO, "ST9"),
		ST10 = new OperationId(typeof(ShortTraversal10), OperationType.SHORT_TRAVERSAL, "ST10"),

		OP1 = new OperationId(typeof(Query1), OperationType.OPERATION_RO, "OP1"),
        OP2 = new OperationId(typeof(Query2), OperationType.OPERATION_RO, "OP2"),
        OP3 = new OperationId(typeof(Query3), OperationType.OPERATION_RO, "OP3"),
        OP4 = new OperationId(typeof(Traversal8), OperationType.OPERATION_RO, "OP4"),
        OP5 = new OperationId(typeof(Traversal9), OperationType.OPERATION_RO, "OP5"),
        OP6 = new OperationId(typeof(Operation6), OperationType.OPERATION_RO, "OP6"),
        OP7 = new OperationId(typeof(Operation7), OperationType.OPERATION_RO, "OP7"),
        OP8 = new OperationId(typeof(Operation8), OperationType.OPERATION_RO, "OP8"),
        OP9 = new OperationId(typeof(Operation9), OperationType.OPERATION, "OP9"),
        OP10 = new OperationId(typeof(Operation10), OperationType.OPERATION, "OP10"),
        OP11 = new OperationId(typeof(Operation11), OperationType.OPERATION, "OP11"),
        OP12 = new OperationId(typeof(Operation12), OperationType.OPERATION, "OP12"),
        OP13 = new OperationId(typeof(Operation13), OperationType.OPERATION, "OP13"),
        OP14 = new OperationId(typeof(Operation14), OperationType.OPERATION, "OP14"),
        OP15 = new OperationId(typeof(Operation15), OperationType.OPERATION, "OP15"),
		
                        


		SM1 = new OperationId(typeof(StructuralModification1), OperationType.STRUCTURAL_MODIFICATION, "SM1"),
        SM2 = new OperationId(typeof(StructuralModification2), OperationType.STRUCTURAL_MODIFICATION, "SM2"),
        SM3 = new OperationId(typeof(StructuralModification3), OperationType.STRUCTURAL_MODIFICATION, "SM3"),
        SM4 = new OperationId(typeof(StructuralModification4), OperationType.STRUCTURAL_MODIFICATION, "SM4"),
        SM5 = new OperationId(typeof(StructuralModification5), OperationType.STRUCTURAL_MODIFICATION, "SM5"),
        SM6 = new OperationId(typeof(StructuralModification6), OperationType.STRUCTURAL_MODIFICATION, "SM6"),
        SM7 = new OperationId(typeof(StructuralModification7), OperationType.STRUCTURAL_MODIFICATION, "SM7"),
        SM8 = new OperationId(typeof(StructuralModification8), OperationType.STRUCTURAL_MODIFICATION, "SM8");

        private static OperationId[] values_ = new OperationId[]{
            T1,T2a,T2b,T2c,T3a,T3b,T3c,T4,T5,T6,Q6,Q7,
            ST1,ST2,ST3,ST4,ST5,ST6,ST7,ST8,ST9,ST10,
            OP1,OP2,OP3,OP4,OP5,OP6,OP7,OP8,OP9,OP10,OP11,OP12,OP13,OP14,OP15,
            SM1,SM2,SM3,SM4,SM5,SM6,SM7,SM8};
        public static OperationId[] values()
        {
            return values_;
        }
        public int ordinal()
        {
            for (int i = 0; i < values_.Length; i++) if (values_[i] == this) return i;
            return -1;
        }

        private Type /*Class<? extends Operation>*/ operationClass;
		private OperationType type;
        private String str;
		
		private OperationId(Type /*Class<? extends Operation>*/ operationClass, OperationType type, String str) {
			this.operationClass = operationClass;
			this.type = type;
            this.str = str;
		}
		
		public OperationType GetType() {
			return type;
		}
		
		public Type /*Class<? extends Operation>*/ GetOperationClass() {
			return operationClass;
		}

        public override String ToString()
        {
            return str;
        }
	}

}
