﻿using System;
 
 
using System.Text;
using STMBench7.Core;
using STMBench7.Backend;
using STMBench7.Annotations;
using STMBench7.Compatability;

namespace STMBench7.Operations
{
	/**
	 * Operation OP7 (see the specification).
	 * Read-only, search on index.
	 */
	public class Operation7 : BaseOperation {

		protected Index/*<IntIndexKey,BaseAssembly>*/ baseAssemblyIdIndex;
		
		public Operation7(Setup oo7setup) {
			this.baseAssemblyIdIndex = oo7setup.GetBaseAssemblyIdIndex();
		}
		
		[Transactional] [ReadOnly]
        //[BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
            
			int baseAssemblyId = ThreadRandom.NextInt(Parameters.MaxBaseAssemblies) +1;
			BaseAssembly baseAssembly = (BaseAssembly)baseAssemblyIdIndex.Get(new IntIndexKey(baseAssemblyId));
			if(baseAssembly == null) throw new OperationFailedException();
			
			ComplexAssembly superAssembly = baseAssembly.GetSuperAssembly();
			
			int count = 0;
			foreach(Assembly siblingAssembly in superAssembly.GetSubAssemblies()) {
				PerformOperationInBaseAssembly((BaseAssembly)siblingAssembly);
				count++;
			}
			
			return count;
		}

        protected virtual void PerformOperationInBaseAssembly(BaseAssembly assembly)
        {
			assembly.NullOperation();
		}
		
        public override OperationId GetOperationId()
        {
	    	return OperationId.OP7;
	    }
	}

}
