﻿using System;
 
 
using System.Text;
using STMBench7.Core;
using STMBench7.Annotations;
using STMBench7.Compatability;
using STMBench7.Backend;

namespace STMBench7.Operations
{

	/**
	 * Operation OP8 (see the specification).
	 * Read-only, search on index.
	 */
	public class Operation8 : BaseOperation {
		
		protected Index/*<IntIndexKey,BaseAssembly>*/ baseAssemblyIdIndex;
		
		public Operation8(Setup oo7setup) {
			this.baseAssemblyIdIndex = oo7setup.GetBaseAssemblyIdIndex();
		}
		
		[Transactional] [ReadOnly]
        //[BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
            
			int baseAssemblyId = ThreadRandom.NextInt(Parameters.MaxBaseAssemblies) +1;
			BaseAssembly baseAssembly = (BaseAssembly)baseAssemblyIdIndex.Get(new IntIndexKey(baseAssemblyId));
			if(baseAssembly == null) throw new OperationFailedException();
			
			int count = 0;
			foreach(CompositePart component in baseAssembly.GetComponents()) {
				PerformOperationInComponent(component);
				count++;
			}
			
			return count;
		}

        protected virtual void PerformOperationInComponent(CompositePart component)
        {
			component.NullOperation();
		}
      
		
        public override OperationId GetOperationId()
        {
	    	return OperationId.OP8;
	    }
	}

}
