﻿using System;
 
 
using System.Text;
using STMBench7.Core;
using STMBench7.Backend;
using STMBench7.Annotations;
using STMBench7.Compatability;

namespace STMBench7.Operations
{
	/**
	 * Query Q2 / Operation OP2 (see the specification).
	 * Read-only, range query on index.
	 */
	public class Query2 : BaseOperation {

	    protected Index/*<IntIndexKey,LargeSet<AtomicPart>>*/ partBuildDateIndex;
	    protected IntIndexKey minAtomicDate, maxAtomicDate;

        public Query2(Setup oo7setup)
            : this(oo7setup, 1)
        {
	    }

	    protected Query2(Setup oo7setup, int percent) {
	    	this.partBuildDateIndex = oo7setup.GetAtomicPartBuildDateIndex();
	    	this.maxAtomicDate = new IntIndexKey(Parameters.MaxAtomicDate);
	    	this.minAtomicDate = 
	    		new IntIndexKey(Parameters.MaxAtomicDate - 
	    				percent * (Parameters.MaxAtomicDate - Parameters.MinAtomicDate) / 100);
	    }

        [BartokSTMExtensions.NoLog]
		[Transactional] [ReadOnly]
        //[BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
            
	    	//Iterable<LargeSet<AtomicPart>> partSets = partBuildDateIndex.GetRange(minAtomicDate, maxAtomicDate);
	    	int count = 0;

	    	//foreach(LargeSet<AtomicPart> partSet in partSets) {
            foreach (LargeSet/*<AtomicPart>*/ partSet in partBuildDateIndex.GetRange(minAtomicDate, maxAtomicDate))
            {
	    	
                foreach(AtomicPart part in partSet) {
	    			PerformOperationInAtomicPart(part);
	    			count++;
	    		}
	    	}

	    	return count;
	    }

        [BartokSTMExtensions.NoLog]
        protected virtual void PerformOperationInAtomicPart(AtomicPart atomicPart)
        {
			atomicPart.NullOperation();
		}
		
        public override OperationId GetOperationId()
        {
	    	return OperationId.OP2;
	    }
	}

}
