﻿using System;
 
 
using System.Text;
using STMBench7.Core;
using STMBench7.Annotations;
using STMBench7.Compatability;
using STMBench7.Backend;

namespace STMBench7.Operations
{
	/**
	 * Query Q5 / Short traversal ST5 (see the specification).
	 * Read-only, iterate on index, short.
	 */
	public class Query5 : BaseOperation {

	    protected Index/*<IntIndexKey,BaseAssembly>*/ baseAssemblyIdIndex;

	    public Query5(Setup oo7setup) {
	    	this.baseAssemblyIdIndex = oo7setup.GetBaseAssemblyIdIndex();
	    }
		
	    [Transactional] [ReadOnly]
        [BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
	    	int result = 0;

	    	foreach(BaseAssembly assembly in baseAssemblyIdIndex) {
	    		result += CheckBaseAssembly(assembly);
	    	}
	    		
	    	return result;
	    }

        [BartokSTMExtensions.NoLog]
        protected virtual int CheckBaseAssembly(BaseAssembly assembly)
        {
	    	int assBuildDate = assembly.GetBuildDate();

	    	foreach(CompositePart part in assembly.GetComponents()) {
	    		if(part.GetBuildDate() > assBuildDate) {
	    			assembly.NullOperation();
	    			return 1;
	    		}
	    	}

	    	return 0;
	    }
	    
        public override OperationId GetOperationId()
        {
	    	return OperationId.ST5;
	    }
	}

}
