﻿using System;
 
 
using System.Text;
using STMBench7.Compatability;
using STMBench7.Core;
using STMBench7.Backend;
using STMBench7.Annotations;

namespace STMBench7.Operations
{
	/**
	 * Short traversal ST1 (see the specification).
	 * Read-only, short.
	 */
	public class ShortTraversal1 : BaseOperation {

		protected Module module;
		
		public ShortTraversal1(Setup oo7setup) {
			this.module = oo7setup.GetModule();
		}

		[Transactional] [ReadOnly]
        [BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
			ComplexAssembly designRoot = module.GetDesignRoot();
			return Traverse(designRoot);
		}

        [BartokSTMExtensions.NoLog]
        protected virtual int Traverse(Assembly assembly)
        {
			if(assembly is ComplexAssembly) return Traverse((ComplexAssembly)assembly);
			else return Traverse((BaseAssembly)assembly);
		}

        [BartokSTMExtensions.NoLog]
        protected virtual int Traverse(ComplexAssembly complexAssembly)
        {
			ImmutableCollection/*<Assembly>*/ subAssemblies = complexAssembly.GetSubAssemblies();
			int numOfSubAssemblies = subAssemblies.Size();
			int nextAssembly = ThreadRandom.NextInt(numOfSubAssemblies);
			
			int subAssemblyNumber = 0;
			foreach(Assembly subAssembly in subAssemblies) {
				if(subAssemblyNumber == nextAssembly) return Traverse(subAssembly);
				subAssemblyNumber++;
			}
			
			throw new RuntimeError("ST1: size of ComplexAssemby.subAssemblies has changed!");
		}

        [BartokSTMExtensions.NoLog]
        protected virtual int Traverse(BaseAssembly baseAssembly)
        {
			ImmutableCollection/*<CompositePart>*/ components = baseAssembly.GetComponents();
			int numOfComponents = components.Size();
			if(numOfComponents == 0) throw new OperationFailedException();
			
			int nextComponent = ThreadRandom.NextInt(numOfComponents);
			
			int componentNumber = 0;
			foreach(CompositePart component in components) {
				if(componentNumber == nextComponent) return Traverse(component);
				componentNumber++;
			}
			
			throw new RuntimeError("ST1: size of BaseAssembly.components has changed!");
		}

        protected virtual int Traverse(CompositePart component)
        {
			LargeSet/*<AtomicPart>*/ atomicParts = component.GetParts();
			int numOfAtomicParts = atomicParts.Size();
			int nextAtomicPart = ThreadRandom.NextInt(numOfAtomicParts);
			
			int atomicPartNumber = 0;
			foreach(AtomicPart atomicPart in atomicParts) {
				if(atomicPartNumber == nextAtomicPart) return Traverse(atomicPart);
				atomicPartNumber++;
			}
			
			throw new RuntimeError("ST1: illegal size of CompositePart.parts!");
		}

        protected virtual int Traverse(AtomicPart atomicPart)
        {
			return atomicPart.GetX() + atomicPart.GetY();
		}
		
        public override OperationId GetOperationId()
        {
	    	return OperationId.ST1;
	    }
	}

}
