﻿using System;
 
 
using System.Text;
using STMBench7.Core;
using STMBench7.Backend;
using STMBench7.Annotations;
using STMBench7.Compatability;

namespace STMBench7.Operations
{
	/**
	 * Structural modification operation SM4 (see the specification).
	 */
	public class StructuralModification4 : BaseOperation {

		protected Index/*<IntIndexKey,BaseAssembly>*/ baseAssemblyIdIndex;
		
		public StructuralModification4(Setup oo7setup) {
			this.baseAssemblyIdIndex = oo7setup.GetBaseAssemblyIdIndex();
		}
		
		[Transactional] [Update]
        [BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
            int baseAssemblyId = ThreadRandom.NextInt(Parameters.MaxBaseAssemblies) + 1;
			BaseAssembly baseAssembly = (BaseAssembly)baseAssemblyIdIndex.Get(new IntIndexKey(baseAssemblyId));
			if(baseAssembly == null) throw new OperationFailedException();
			
			ImmutableCollection/*<CompositePart>*/ components = baseAssembly.GetComponents();
			int numOfComponents = components.Size();
			if(numOfComponents == 0) throw new OperationFailedException();
			
			int componentToRemove = ThreadRandom.NextInt(numOfComponents);
			
			int componentNumber = 0;
			foreach(CompositePart component in components) {
				if(componentNumber == componentToRemove) {
					baseAssembly.RemoveComponent(component);
					return 0;
				}
				componentNumber++;
			}
			
			throw new RuntimeError("SM4: concurrent modification of BaseAssembly.components!");
            
		}
		
        public override OperationId GetOperationId()
        {
	    	return OperationId.SM4;
	    }
	}
}
