﻿using System;
 
 
using System.Text;
using STMBench7.Core;
using STMBench7.Backend;
using STMBench7.Annotations;
using STMBench7.Compatability;

namespace STMBench7.Operations
{
	/**
	 * Structural modification operation SM5 (see the specification).
	 */
	public class StructuralModification5 : BaseOperation {

		protected AssemblyBuilder assemblyBuilder;
		protected Index/*<IntIndexKey,BaseAssembly>*/ baseAssemblyIdIndex;
		protected Module module;
		
		public StructuralModification5(Setup oo7setup) {
			this.baseAssemblyIdIndex = oo7setup.GetBaseAssemblyIdIndex();
			assemblyBuilder = oo7setup.GetAssemblyBuilder();
			this.module = oo7setup.GetModule();
		}
		
		[Transactional] [Update]
        [BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
            int siblingBaseAssemblyId = ThreadRandom.NextInt(Parameters.MaxBaseAssemblies) + 1;
			BaseAssembly siblingBaseAssembly = (BaseAssembly)baseAssemblyIdIndex.Get(new IntIndexKey(siblingBaseAssemblyId));
			if(siblingBaseAssembly == null) throw new OperationFailedException();
			
			ComplexAssembly superAssembly = siblingBaseAssembly.GetSuperAssembly();
			assemblyBuilder.CreateAndRegisterAssembly(module, superAssembly);
			
            return 0;
		}

        public override OperationId GetOperationId()
        {
	    	return OperationId.SM5;
	    }
	}

}
