﻿using System;
 
 
using System.Text;
using STMBench7.Annotations;
using STMBench7.Core;
using STMBench7.Backend;
using STMBench7.Compatability;

namespace STMBench7.Operations
{	
	/**
	 * Structural modification SM6 (see the specification).
	 */
	public class StructuralModification6 : StructuralModification5 {

		protected Index/*<IntIndexKey, BaseAssembly>*/ baseAssemblyIdIndex;
		
		public StructuralModification6(Setup oo7setup) : base(oo7setup) {
			this.baseAssemblyIdIndex = oo7setup.GetBaseAssemblyIdIndex();
		}

		[Transactional] [Update]
        [BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
            int baseAssemblyToRemoveId = ThreadRandom.NextInt(Parameters.MaxBaseAssemblies) + 1;
			BaseAssembly baseAssemblyToRemove = (BaseAssembly)baseAssemblyIdIndex.Get(new IntIndexKey(baseAssemblyToRemoveId));
			if(baseAssemblyToRemove == null) throw new OperationFailedException();
			
			// We want the tree of BAs/CAs to keep its form 
			// so that each CA has always at least one child sub-assembly
			if(baseAssemblyToRemove.GetSuperAssembly().GetSubAssemblies().Size() == 1)
				throw new OperationFailedException();
			
			assemblyBuilder.UnregisterAndRecycleBaseAssembly(baseAssemblyToRemove);
			
			return 1;
		}
		
        public override OperationId GetOperationId()
        {
	    	return OperationId.SM6;
	    }
	}

}
