﻿using System;
 
 
using System.Text;
using STMBench7.Compatability;
using STMBench7.Annotations;
using STMBench7.Core;

namespace STMBench7.Operations
{
	/**
	 * Structural modification SM8 (see the specification).
	 */
	public class StructuralModification8 : StructuralModification7 {

        public StructuralModification8(Setup oo7setup)
            : base(oo7setup)
        {
		}

		[Transactional] [Update]
        [BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
            int complexAssemblyId = ThreadRandom.NextInt(Parameters.MaxComplexAssemblies) + 1;
			ComplexAssembly complexAssembly = (ComplexAssembly)complexAssemblyIdIndex.Get(new IntIndexKey(complexAssemblyId));
			if(complexAssembly == null) throw new OperationFailedException();

			// We want the tree of BAs/CAs to keep its form 
			// so that each CA has always at least one child sub-assembly
			ComplexAssembly superAssembly = complexAssembly.GetSuperAssembly();
			if(superAssembly == null || superAssembly.GetSubAssemblies().Size() == 1)
				throw new OperationFailedException();
			
			assemblyBuilder.UnregisterAndRecycleComplexAssembly(complexAssembly);
			
			return 1;
		}
		
        public override OperationId GetOperationId()
        {
	    	return OperationId.SM8;
	    }
	}

}
