﻿using System;
 
 
using System.Text;
using STMBench7.Core;
using STMBench7.Compatability;
using STMBench7.Backend;
using STMBench7.Annotations;

namespace STMBench7.Operations
{
	/**
	 * Traversal T7 / Short traversal ST3 (see the specification).
	 * Read-only, search on index, short.
	 */
	public class Traversal7 : BaseOperation {

	    Index/*<IntIndexKey,AtomicPart>*/ partIdIndex;

	    public Traversal7(Setup oo7setup) {
	    	this.partIdIndex = oo7setup.GetAtomicPartIdIndex();
	    }

	    [Transactional] [ReadOnly]
        [BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
            
            IntIndexKey partId = new IntIndexKey(ThreadRandom.NextInt(Parameters.MaxAtomicParts) + 1);
	    	AtomicPart part = (AtomicPart)partIdIndex.Get(partId);
	    	if(part == null) throw new OperationFailedException();
	    	
	    	return Traverse(part.GetPartOf());
             
	    }
        
        protected virtual int Traverse(CompositePart part)
        {
           HashtableSet/*<Assembly>*/ visitedAssemblies = new HashtableSet/*<Assembly>*/();
        
	    	int result = 0;
	    	ImmutableCollection/*<BaseAssembly>*/ ownerBaseAssemblies = part.GetUsedIn();
	    	foreach(BaseAssembly assembly in ownerBaseAssemblies)
	    		result += Traverse(assembly, visitedAssemblies);
        
	    	return result;
	    }

       protected virtual int Traverse(Assembly assembly, HashtableSet/*<Assembly>*/ visitedAssemblies)
        {
	    	if(assembly == null) return 0;
	    	if(visitedAssemblies.Contains(assembly)) return 0;

	    	visitedAssemblies.Add(assembly);

	    	PerformOperationOnAssembly(assembly);
	    	
	    	return Traverse(assembly.GetSuperAssembly(), visitedAssemblies) + 1;
	    }

        protected virtual void PerformOperationOnAssembly(Assembly assembly)
        {
	    	assembly.NullOperation();
	    }
	    
        public override OperationId GetOperationId()
        {
	    	return OperationId.ST3;
	    }
	}

}
