﻿using System;

using System.Text;

using STMBench7.Core;
using STMBench7.Annotations;
using STMBench7.Backend;
using STMBench7.Operations;

namespace STMBench7
{
	/**
	 * Sets up the benchmark structures according to given parameters,
	 * including indexes.
	 */
	[Immutable]
	public class Setup {
	    
	    protected Module module;

	    protected Index/*<IntIndexKey,AtomicPart>*/ atomicPartIdIndex;
	    protected Index/*<IntIndexKey,LargeSet<AtomicPart>>*/ atomicPartBuildDateIndex;
	    protected Index/*<StringIndexKey,Document>*/ documentTitleIndex;
	    protected Index/*<IntIndexKey,CompositePart>*/ compositePartIdIndex;
	    protected Index/*<IntIndexKey,BaseAssembly>*/ baseAssemblyIdIndex;
        protected Index/*<IntIndexKey,ComplexAssembly>*/ complexAssemblyIdIndex;

	    protected CompositePartBuilder compositePartBuilder;
	    protected ModuleBuilder moduleBuilder;
	    
	    public Setup() {
	    	BackendFactory backendFactory = BackendFactory.instance;
	    	
	    	atomicPartIdIndex = backendFactory.CreateIndex/*<IntIndexKey,AtomicPart>*/();
	    	atomicPartBuildDateIndex = backendFactory.CreateIndex/*<IntIndexKey,LargeSet<AtomicPart>>*/();
	    	documentTitleIndex = backendFactory.CreateIndex/*<StringIndexKey,Document>*/();
	    	compositePartIdIndex = backendFactory.CreateIndex/*<IntIndexKey,CompositePart>*/();
	    	baseAssemblyIdIndex = backendFactory.CreateIndex/*<IntIndexKey,BaseAssembly>*/();
            complexAssemblyIdIndex = backendFactory.CreateIndex/*<IntIndexKey, ComplexAssembly>*/();    	

	    	compositePartBuilder = new CompositePartBuilder(compositePartIdIndex,
	    			documentTitleIndex,	atomicPartIdIndex, atomicPartBuildDateIndex);
	    	moduleBuilder = new ModuleBuilder(baseAssemblyIdIndex, complexAssemblyIdIndex);

	    	SetupDataStructure setupOperation = new SetupDataStructure(this);
            
            try {
	    		OperationExecutorFactory.ExecuteSequentialOperation(setupOperation);
            }
	    	catch(OperationFailedException) {
	    		throw new RuntimeError("Unexpected failure of the setup operation");
	    	}

	    	module = setupOperation.GetModule();
	    }
	    
		public Index/*<IntIndexKey, LargeSet<AtomicPart>>*/ GetAtomicPartBuildDateIndex() {
			return atomicPartBuildDateIndex;
		}

		public Index/*<IntIndexKey, AtomicPart>*/ GetAtomicPartIdIndex() {
			return atomicPartIdIndex;
		}

		public Index/*<IntIndexKey, BaseAssembly>*/ GetBaseAssemblyIdIndex() {
			return baseAssemblyIdIndex;
		}

        public Index/*<IntIndexKey, ComplexAssembly>*/ GetComplexAssemblyIdIndex()
        {
			return complexAssemblyIdIndex;
		}

		public Index/*<IntIndexKey, CompositePart>*/ GetCompositePartIdIndex() {
			return compositePartIdIndex;
		}

        public Index/*<StringIndexKey, Document>*/ GetDocumentTitleIndex()
        {
			return documentTitleIndex;
		}

		public Module GetModule() {
			return module;
		}
		
		public CompositePartBuilder GetCompositePartBuilder() {
			return compositePartBuilder;
		}
		
		public ModuleBuilder GetModuleBuilder() {
			return moduleBuilder;
		}
		
		public AssemblyBuilder GetAssemblyBuilder() {
			return moduleBuilder.GetAssemblyBuilder();
		}
	}
}
